Authenticated = "Authenticated"
Allow = "Allow"
Deny = "Deny"
Everyone = "Everyone"

ACE = tuple[str, str, str]
ACL = tuple[ACE, ...]
DEFAULT_ACL: ACL = ((Allow, Everyone, "public"),)


class ACLProvider:
    acl: ACL = ()

    def __init__(self, acl: ACL = ()) -> None:
        self.acl = acl

    def __acl__(self) -> ACL:
        return self.acl + DEFAULT_ACL



import logging
from http import HTTPStatus
from typing import Optional

from fastapi import HTTPException
from fastapi.requests import HTTPConnection

from coderfastapi.lib.security import Allow, Deny, Everyone
from coderfastapi.lib.security.acl import ACL, ACLProvider

log = logging.getLogger(__name__)


class AuthorizationPolicy:
    acl_provider: ACLProvider

    def __init__(self, acl_provider: ACLProvider) -> None:
        self.acl_provider = acl_provider

    def validate_permission(
        self,
        requested_permission: str,
        http_connection: HTTPConnection,
        context_acl_provider: Optional[ACLProvider] = None,
    ) -> None:
        try:
            allowed = self.check_permission(
                requested_permission, http_connection, context_acl_provider
            )
        except ValueError as e:
            log.exception(e)
            allowed = False

        if allowed:
            return

        raise HTTPException(
            status_code=HTTPStatus.FORBIDDEN,
            detail="Permission denied.",
        )

    def check_permission(
        self,
        requested_permission: str,
        http_connection: HTTPConnection,
        context_acl_provider: Optional[ACLProvider] = None,
    ) -> bool:
        principals = self.get_principals(http_connection)

        for action, principal, permission in self.get_complete_acl(
            context_acl_provider
        ):
            if permission != requested_permission or principal not in principals:
                continue

            if action is Allow:
                log.info(f"Permission '{permission}' granted")
                return True
            elif action is Deny:
                log.info(f"Permission '{permission}' denied")
                return False
            else:
                raise ValueError("Invalid action in ACL")

        return False

    @classmethod
    def get_principals(cls, http_connection: HTTPConnection) -> tuple[str, ...]:
        return (Everyone,)

    def get_complete_acl(
        self,
        context_acl_provider: Optional[ACLProvider] = None,
    ) -> ACL:
        acl = self.acl_provider.__acl__()
        if not context_acl_provider:
            return acl

        complete_acl = context_acl_provider.__acl__() + acl
        log.debug(f"complete ACL: {complete_acl}")

        return complete_acl



from fastapi import APIRouter, FastAPI

from coderfastapi.lib.validation.schemas import VersionSchema


def register_version_handler(
    router: APIRouter | FastAPI,
    version: str,
    path: str = "/",
) -> None:
    @router.get(path, response_model=VersionSchema)
    async def get_version():
        return {"version": version}



import copy
from inspect import Parameter, Signature
from typing import Sequence


def copy_parameters(
    from_signature: Signature,
    to_signature: Signature,
    parameter_keys: Sequence[str],
) -> Signature:
    filtered_parameters = [
        to_signature.parameters[key]
        for key in to_signature.parameters
        if key not in parameter_keys
    ]

    new_parameters = _bulk_create_new_parameters(
        filtered_parameters, [from_signature.parameters[key] for key in parameter_keys]
    )
    new_signature = to_signature.replace(parameters=new_parameters)
    return new_signature


def _bulk_create_new_parameters(
    existing_parameters: list[Parameter],
    new_parameters: list[Parameter],
) -> list[Parameter]:
    parameters = existing_parameters
    for new_parameter in new_parameters:
        parameters = _create_new_parameters(parameters, new_parameter)
    return parameters


def _create_new_parameters(
    existing_parameters: list[Parameter],
    new_parameter: Parameter,
) -> list[Parameter]:
    if not existing_parameters:
        return [new_parameter]

    if check_parameter_has_default(new_parameter):
        new_parameters = copy.copy(existing_parameters)
        for i, parameter in enumerate(new_parameters):
            if check_parameter_has_default(parameter):
                new_parameters.insert(i, new_parameter)
                return new_parameters

        return existing_parameters + [new_parameter]
    else:
        return [new_parameter] + existing_parameters


def check_parameter_has_default(parameter: Parameter) -> bool:
    return parameter.default != Parameter.empty



import logging
from typing import Any
from uuid import UUID

from coderfastapi.lib.security.policies.authentication.jwt.providers import (
    JWTDataProvider,
    T,
)

log = logging.getLogger(__name__)


class UserDataProvider(JWTDataProvider):
    def augment_request(self, request: T, data: dict[str, Any]) -> T:
        request_ = super().augment_request(request, data)
        try:
            request_.user_id = UUID(data["user_id"])
            log.info(f"Authenticated user: {request_.user_id}")
        except (KeyError, ValueError):
            request_.user_id = None

        return request_

    def parse_to_encode(self, data: dict[str, Any]) -> dict[str, Any]:
        return {"user_id": str(data["user_id"])}
